/*
 * Data structure containing all parameters needed by a pointcut checker
 * when seeing if a given policy input (commands, context, etc.)
 * matches a given pointcut.
 */

package rewrite.pointcut;

import java.util.*;
import java.util.regex.*;

import rewrite.util.*;

import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import org.apache.bcel.Repository;
import org.apache.bcel.util.*;
import org.apache.bcel.Constants;

import spox.policy.pointcut.PointcutComponent.*;
import spox.policy.pointcut.*;
import spox.policy.*;

/**
 * Written by Micah Jones
 */
public class PointcutData
{
//
//    public final Policy policy;
//    
//    // Constructor
//    public PointcutData(Policy policy)
//    { this.policy = policy; }
    
    public Integer modifiers = null;
    public Type classType = null;
    public String memberName = null;
    public Type retType = null;
    public Type[] argType = null;
    public Type[] throwsType = null;
    
    public ClassGen cgen = null;
    public ConstantPoolGen cpgen = null;
    public Method method = null;
    public MethodGen mgen = null;
    public Instruction instruction = null;
    public InstructionHandle ih = null;

    /**
     * A list of all classes that have instance states attached.
     */
    public ArrayList<String> stateObj = new ArrayList<String>();
    /**
     * A mapping from an edge's instance variable id (e.g., 'obj="x"') to the
     * actual, fully qualified name of the object (e.g., java.lang.Object).
     */
    public HashMap<String, String> nodeObj = new HashMap<String, String>();
    public HashMap<String, Integer> nodeObjIndex = new HashMap<String, Integer>();
    
    /**
     * Tells the rewriter that an instance id variable is tied to the 'this' pointer.
     */
    public HashMap<String, Boolean> thisObj = new HashMap<String, Boolean>();

    public boolean generateCode = true;
    
    // All of the following fields should be modified only be the pointcut
    // matcher class as return data for the rewriter
    public InstructionList returnIlist =
            new InstructionList(InstructionFactory.NOP);
    public InstructionBlock iBlock;
    
    // Temp field for dynamic code insertion purposes; be careful about use
    public enum BopContainer {and, or, not};
    //public BopContainer bopContainer = BopContainer.and;
    
    // Inner classes
    public static class InstructionBlock
    {
        // Contains an InstructionList and possibly other InstructionBlocks
        // This allows us to nest ANDs and ORs together by storing each
        // in its own block
        public BopContainer bopContainer = null; // A null marks the base case
        public InstructionList ilist = new InstructionList();
        public ArrayList<InstructionBlock> iBlockArray =
                new ArrayList<InstructionBlock>();
        
        // Constructor
        public InstructionBlock()
        {
            // Initializes the instruction list
            ilist.append(InstructionFactory.NOP); // A true branch
            ilist.append(InstructionFactory.NOP); // A false branch
        }
    }
    
}
