/*
 * Ensures uniqueness of various names for files, methods, etc.
 * 
 * All methods are static.
 */

package rewrite.util;

import spox.policy.PolicyComponent.*;

import java.io.*;
import java.util.*;
import java.util.jar.*;

import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import org.apache.bcel.util.*;
import spox.policy.PolicyComponent;

public class UniqueNameGen
{

    public UniqueNameGen() {}

    
    public static String findUniqueFieldName
            (ClassGen cgen, String baseName)
    {
        // Scans a class's fields to see if baseName conflicts with any
        // If not, it just returns baseName
        // Is so, it appends an incrementing number to the end until
        // a distinct name is found
        
        Field[] fields = cgen.getFields();
        String[] fieldNames = new String[fields.length];
        String name = baseName;
        int n = 2;
        
        // Generate fieldNames so we have strings to compare to
        for (int i = 0; i < fields.length; i++)
            fieldNames[i] = fields[i].getName();
            
        while (Misc.arrayContains(fieldNames, name) && n < Integer.MAX_VALUE)
        {
            // Tack on an incrementing number until something works
            name = baseName + n;
            n = n + 1;
        }
        
        return name;
        
    }
    
    public static String findUniqueMethodName
            (ClassGen cgen, String baseName)
    {
        // Scans a class's methods to see if baseName conflicts with any
        // If not, it just returns baseName
        // Is so, it appends an incrementing number to the end until
        // a distinct name is found
        
        Method[] methods = cgen.getMethods();
        String[] methodNames = new String[methods.length];
        String name = baseName;
        int n = 2;
        
        // Generate methodNames so we have strings to compare to
        for (int i = 0; i < methods.length; i++)
            methodNames[i] = methods[i].getName();
            
        while (Misc.arrayContains(methodNames, name) && n < Integer.MAX_VALUE)
        {
            // Tack on an incrementing number until something works
            name = baseName + n;
            n = n + 1;
        }
        
        return name;
        
    }

    public static String findUniqueLocalVariableName
            (MethodGen mgen, String baseName)
    {
        // Scans a method's local variables to see if baseName conflicts with any
        // If not, it just returns baseName
        // Is so, it appends an incrementing number to the end until
        // a distinct name is found
        
        LocalVariableGen[] lvgen = mgen.getLocalVariables();
        String[] lvNames = new String[lvgen.length];
        String name = baseName;
        int n = 2;
        
        // Generate lvNames so we have strings to compare to
        for (int i = 0; i < lvgen.length; i++)
            lvNames[i] = lvgen[i].getName();
            
        while (Misc.arrayContains(lvNames, name) && n < Integer.MAX_VALUE)
        {
            // Tack on an incrementing number until something works
            name = baseName + n;
            n = n + 1;
        }
        
        return name;
        
    }

    public static String findUniqueComponentName
            (ArrayList<Component> component, String baseName)
    {
        // Scans a component arraylist to see if baseName conflicts with any
        // If not, it just returns baseName
        // Is so, it appends an incrementing number to the end until
        // a distinct name is found
        
        String name = baseName;
        int n = 2;
        
        while (Misc.componentArrayContainsName(component,name) && n < Integer.MAX_VALUE)
        {
            // Tack on an incrementing number until something works
            name = baseName + n;
            n = n + 1;
        }
        
        return name;
        
    }

    public static String findUniqueClassFileName
            (JarFile jarfile, String baseName)
    {
        // Scans a jar file's classes to see if baseName conflicts with any
        // If not, it just returns baseName
        // Is so, it appends an incrementing number to the end until
        // a distinct name is found
        
        String name = baseName;
        int n = 2;
        
        while (jarfile.getEntry(name+".class") != null && n < Integer.MAX_VALUE)
        {
            // Tack on an incrementing number until something works
            name = baseName + n;
            n = n + 1;
        }
        
        return name;
        
    }

    public static String findUniqueManifestAttributeName
            (Manifest manifest, String baseName)
    {
        // Scans a manifest's attributes to see if baseName conflicts with any
        // If not, it just returns baseName
        // Is so, it appends an incrementing number to the end until
        // a distinct name is found
        
        String name = baseName;
        int n = 2;
        Attributes attributes = manifest.getMainAttributes();
        
        while (attributes.getValue(name) != null && n < Integer.MAX_VALUE)
        {
            // Tack on an incrementing number until something works
            name = baseName + n;
            n = n + 1;
        }
        
        return name;
        
    }

    public static String findUniqueEdgeName(Root root, String baseName)
    {
        ArrayList<Edge> edge = root.edge;
        for (int i = 0; i < root.forall.size(); i++)
            edge.addAll(PolicyComponent.extractEdges(root.forall.get(i)));
        
        ArrayList<Component> component = new ArrayList<Component>();
        for (int i = 0; i < edge.size(); i++)
            component.add(edge.get(i));
        
        return findUniqueComponentName(component, baseName);
    }
    
}
